@@ -152,7 +152,7 @@ t_cached_object *add_to_cache(t_session *session, char *file) {
 	object->last_changed = status.st_mtime;
 	object->timer = TIME_IN_CACHE + delay_timer;
 	object->size = size;
-	object->busy = 1;
+	object->in_use = 1;
 	object->last_ip = session->ip_address;
 
 	i = cache_index(file);
@@ -199,12 +199,12 @@ t_cached_object *search_cache(t_session *session, char *file) {
 							}
 							object->last_ip = session->ip_address;
 						}
-						(object->busy)++;
+						(object->in_use)++;
 						result = object;
-					} else if (object->busy <= 0) {
+					} else if (object->in_use <= 0) {
 						remove_from_cache(object, i);
 					}
-				} else if (object->busy <= 0) {
+				} else if (object->in_use <= 0) {
 					remove_from_cache(object, i);
 				}
 				break;
@@ -219,7 +219,7 @@ t_cached_object *search_cache(t_session *session, char *file) {
 }
 
 void done_with_cached_object(t_cached_object *object) {
-	(object->busy)--;
+	(object->in_use)--;
 }
 
 void check_cache(void) {
@@ -233,7 +233,7 @@ void check_cache(void) {
 			object = cache[i];
 			while (object != NULL) {
 				if ((object->timer -= MAX_CACHE_DELAY_TIMER) <= 0) {
-					if (object->busy <= 0) {
+					if (object->in_use <= 0) {
 						object = remove_from_cache(object, i);
 						continue;
 					} else {
@@ -292,7 +292,7 @@ int clear_cache(void) {
 			object = list;
 			list = list->next;
 
-			if (object->busy == 0) {
+			if (object->in_use == 0) {
 				/* Object unused, so remove
 				 */
 				cachesize -= object->size;
@@ -303,7 +303,7 @@ int clear_cache(void) {
 
 				removed++;
 			} else {
-				/* Object busy, put back in list
+				/* Object in use, put back in list
 				 */
 				object->next = cache[i];
 				cache[i] = object;
