@@ -1,6 +1,9 @@
 #ifndef _CACHE_H
 #define _CACHE_H
 
+#include "config.h"
+#include "global.h"
+
 #define TIME_IN_CACHE     60
 #define MAX_CACHE_TIMER 3600
 
@@ -10,8 +13,8 @@ typedef struct type_cached_object {
 	off_t         size;
 	int           timer;
 	time_t        last_changed;
-	volatile int  busy;
-	long          last_ip;
+	volatile int  in_use;
+	t_ipv4        last_ip;
 
 	struct type_cached_object *prev;
 	struct type_cached_object *next;
