@@ -47,10 +47,10 @@ void print_code(int code) {
 }
 
 void log_error(char *str) {
-	fprintf(stderr, "CGI-wrapper: %s", str);
+	fprintf(stderr, "CGI-wrapper: %s\n", str);
 }
 
-bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
+bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data, char *user_directory) {
 	FILE *fp;
 	char line[257], *item, *rest, *pipe;
 	struct passwd *pwd;
@@ -71,11 +71,11 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
 			return false;
 		}
 		len = strlen(pwd->pw_dir);
-		if ((wrap_data->cgiroot = (char*)malloc(len + 14)) == NULL) {
+		if ((wrap_data->cgiroot = (char*)malloc(len + strlen(user_directory) + 1)) == NULL) {
 			return false;
 		}
 		memcpy(wrap_data->cgiroot, pwd->pw_dir, len);
-		memcpy(wrap_data->cgiroot + len, "/public_html/\0", 14);
+		strcpy(wrap_data->cgiroot + len, user_directory);
 		if (strncmp(wrap_data->cgiroot, cgi, strlen(wrap_data->cgiroot)) != 0) {
 			return false;
 		}
@@ -91,7 +91,7 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
 	} else {
 		wrap_data->cgiroot = NULL;
 	}
-	
+
 	/* Read CGI wrapper configuration
 	 */
 	if ((fp = fopen(CONFIG_DIR"/cgi-wrapper.conf", "r")) != NULL) {
@@ -150,11 +150,11 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
 							*pipe = '/';
 						}
 
-						len = strlen(item);
-						if (strncmp(item, cgi, len) != 0) {
+						if ((len = strlen(item)) == 0) {
 							break;
 						}
-						if ((*(item + len - 1) != '/') && (*(cgi + len) != '/')) {
+						if ((strncmp(item, cgi, len) != 0) || (*(cgi + len) != '/')) {
+							log_error("CGI not in WebsiteRoot");
 							break;
 						}
 
@@ -166,18 +166,21 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
 						wrap_data->cgiroot = strdup(item);
 					} else if (*item == '~') {
 						if ((pwd = getpwnam(item + 1)) == NULL) {
+							log_error("invalid username");
 							break;
 						}
 						len = strlen(pwd->pw_dir);
-						if ((wrap_data->cgiroot = (char*)malloc(len + 14)) == NULL) {
+						if ((wrap_data->cgiroot = (char*)malloc(len + strlen(user_directory) + 1)) == NULL) {
 							break;
 						}
 						memcpy(wrap_data->cgiroot, pwd->pw_dir, len);
-						memcpy(wrap_data->cgiroot + len, "/public_html/\0", 14);
+						strcpy(wrap_data->cgiroot + len, user_directory);
 						if (strncmp(wrap_data->cgiroot, cgi, strlen(wrap_data->cgiroot)) != 0) {
+							log_error("CGI not in user directory");
 							break;
 						}
 					} else {
+						log_error("invalid CGI root");
 						break;
 					}
 
@@ -185,6 +188,7 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
 					 */
 					split_string(rest, &item, &rest, ':');
 					if (parse_userid(item, &(wrap_data->uid)) != 1) {
+						log_error("invalid userid");
 						break;
 					}
 
@@ -192,10 +196,12 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
 					 */
 					if (rest != NULL) {
 						if (parse_groups(rest, &(wrap_data->gid), &(wrap_data->groups)) != 1) {
+							log_error("syntax error in groupid");
 							break;
 						}
 					} else {
 						if (lookup_group_ids(wrap_data->uid, &(wrap_data->gid), &(wrap_data->groups)) != 1) {
+							log_error("invalid group (user member of root?)");
 							break;
 						}
 					}
@@ -232,15 +238,16 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data) {
 }
 
 int main(int argc, char *argv[]) {
-	char buffer[8], *wrapid, *var;
+	char buffer[8], *wrapid, *var, *user_directory;
 	FILE *fp;
 	int i, handle, arg_offset, time_for_cgi, len;
 	t_wrap wrap_data;
 	bool follow_symlinks = true, usecgi_handler;
 
-	char *cgiwrap_id   = "CGIWRAP_ID";
-	char *cgiwrap_sln  = "CGIWRAP_FOLLOWSYMLINKS";
-	char *cgiwrap_time = "CGIWRAP_TIMEFORCGI";
+	char *cgiwrap_id      = "CGIWRAP_ID";
+	char *cgiwrap_symlink = "CGIWRAP_FOLLOWSYMLINKS";
+	char *cgiwrap_cgitime = "CGIWRAP_TIMEFORCGI";
+	char *cgiwrap_userdir = "CGIWRAP_USERDIRECTORY";
 
 	if (argc < 3) {
 		print_code(-1);
@@ -278,23 +285,33 @@ int main(int argc, char *argv[]) {
 		print_code(500);
 	}
 
-	if ((var = getenv(cgiwrap_sln)) == NULL) {
+	if ((var = getenv(cgiwrap_symlink)) == NULL) {
 		log_error("getenv(CGIWRAP_FOLLOWSYMLINKS) error");
 		print_code(500);
 	}
 	follow_symlinks = (strcmp(var, "true") == 0);
 
-	if ((var = getenv(cgiwrap_time)) != NULL) {
+	if ((var = getenv(cgiwrap_cgitime)) != NULL) {
 		time_for_cgi = str2int(var);
 	} else {
 		time_for_cgi = 5;
 	}
 
+	if ((var = getenv(cgiwrap_userdir)) != NULL) {
+		if ((user_directory = (char*)malloc(strlen(var) + 3)) == NULL) {
+			print_code(500);
+		}
+		sprintf(user_directory, "/%s/", var);
+	} else {
+		user_directory = "/public_html/";
+	}
+
 	/* Clear environment crap
 	 */
 	unsetenv(cgiwrap_id);
-	unsetenv(cgiwrap_sln);
-	unsetenv(cgiwrap_time);
+	unsetenv(cgiwrap_symlink);
+	unsetenv(cgiwrap_cgitime);
+	unsetenv(cgiwrap_userdir);
 
 	/* Check for bad path
 	 */
@@ -306,11 +323,15 @@ int main(int argc, char *argv[]) {
 	/* Read cgi-wrapper config
 	 */
 	if ((usecgi_handler = (strcmp(argv[0], "-") != 0))) {
-		if (get_wrap_data(wrapid, argv[0], argv[2], &wrap_data) == false) {
+		/* CGI handler
+		 */
+		if (get_wrap_data(wrapid, argv[0], argv[2], &wrap_data, user_directory) == false) {
 			print_code(403);
 		}
 	} else {
-		if (get_wrap_data(wrapid, NULL, argv[2], &wrap_data) == false) {
+		/* CGI program
+		 */
+		if (get_wrap_data(wrapid, NULL, argv[1], &wrap_data, user_directory) == false) {
 			print_code(403);
 		}
 	}
@@ -362,10 +383,10 @@ int main(int argc, char *argv[]) {
 	 */
 	if ((handle = open(argv[2], O_RDONLY)) == -1) {
 		if (errno == EACCES) {
-			log_error("access to CGIhandler denied");
+			log_error("access to CGI program denied");
 			print_code(403);
 		}
-		log_error("CGIhandler does not exist");
+		log_error("CGI program does not exist");
 		print_code(404);
 	} else {
 		close(handle);
@@ -374,7 +395,7 @@ int main(int argc, char *argv[]) {
 	/* Symlink allowed?
 	 */
 	if (follow_symlinks == false) {
-		switch (contains_not_allowed_symlink(argv[1], wrap_data.cgiroot)) {
+		switch (contains_not_allowed_symlink(argv[2], wrap_data.cgiroot)) {
 			case error:
 				log_error("contains_not_allowed_symlink() error");
 				print_code(500);
