@@ -16,6 +16,7 @@
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#include "global.h"
 #include "libstr.h"
 #include "send.h"
 #include "cgi.h"
@@ -23,7 +24,6 @@
 #include "log.h"
 
 #define MAX_CGI_DELAY_TIMER 60
-#define ip_index(ip) (ip & 255)
 #define DUMMY_BUFFER_SIZE 65536
 
 static int delay_timer = 0;
@@ -45,7 +45,7 @@ void init_load_balancer(t_fcgi_server *fcgi_server) {
 	}
 }
 
-t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, unsigned long client_ip) {
+t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ipv4 client_ip) {
 	t_connect_to  *connect_to = NULL;
 	t_cgi_session *cgi_session;
 	bool search_new_fcgi_server = true;
@@ -184,7 +184,7 @@ t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi
 		fcgi = fcgi_server;
 		while (fcgi != NULL) {
 			if (strcmp(fastcgi->item[i], fcgi->fcgi_id) == 0) {
-				if (in_charlist(extension, &(fcgi_server->extension))) {
+				if (in_charlist(extension, &(fcgi->extension))) {
 					return fcgi;
 				}
 			}
@@ -266,6 +266,9 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 				cgi_time[15] = '\0';
 				snprintf(cgi_time, 15, "%d", session->config->time_for_cgi);
 				setenv("CGIWRAP_TIMEFORCGI", cgi_time, 1);
+				if (session->config->user_directory_set) {
+					setenv("CGIWRAP_USERDIRECTORY", session->config->user_directory, 1);
+				}
 			} else if (setsid() == -1) {
 				exit(EXIT_FAILURE);
 			}
@@ -384,8 +387,8 @@ int read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 }
 
 /*
- *  Fast CGI processes
- *  ===================
+ *  FastCGI servers
+ *  ================
  */
 
 int connect_to_fcgi_server(t_connect_to *connect_to) {
