@@ -54,14 +54,25 @@ typedef struct type_cgi_info {
 	int fcgi_data_len;
 } t_cgi_info;
 
+typedef struct type_fcgi_program {
+	char  *filename;
+	pid_t pid;
+	int   to_cgi, from_cgi, cgi_error;
+	int   runs, timer;
+	volatile int in_use;
+} t_fcgi_program;
+
+// FastCGI server loadbalancer
 void init_load_balancer(t_fcgi_server *fcgi_server);
-t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, unsigned long client_ip);
+t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ipv4 client_ip);
 t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi, char *cgi_script);
 void check_load_balancer(t_config *config);
 
+// Normal CGI
 pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info);
 int read_from_cgi_process(t_session *session, t_cgi_info *cgi_info);
 
+// FastCGI server
 int connect_to_fcgi_server(t_connect_to *connect_to);
 int send_fcgi_request(t_session *session, int sock);
 int read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info);
