@@ -13,6 +13,7 @@
 #include <string.h>
 #include <pthread.h>
 #include <time.h>
+#include "global.h"
 #include "libstr.h"
 #include "libmd5.h"
 #include "command.h"
@@ -25,9 +26,6 @@
 #define MAX_CMD_SIZE   64
 #define TIMESTAMP_SIZE 50
 
-#define KILOBYTE    1024
-#define MEGABYTE 1048576
-
 static t_admin *adminlist, *loop, *next;
 static char *prompt = "\033[01;34mhcc>\033[00m ";
 
@@ -242,7 +240,7 @@ static void show_status(FILE *fp) {
 
 static int run_command(char *line, FILE *fp, t_config *config) {
 	char *cmd, *param, *param2;
-	unsigned long ip;
+	t_ipv4 ip;
 	int retval = 0, time, id, count;
 	
 	split_string(line, &cmd, &param, ' ');
