@@ -1,36 +1,13 @@
-/* Hiawatha | session.c
- *
- * All the routines to handle a session. A session is a connection from a
- * client to the webserver.
+/* Hiawatha | envir.c
  */
 #include "config.h"
 #include <stdlib.h>
 #include <string.h>
 #include "envir.h"
+#include "session.h"
 
-#ifndef HAVE_SETENV
-int setenv(const char *key, const char *value, int overwrite) {
-	char *line;
+#define SSL_VAR_SIZE 512
 
-	if ((getenv(key) == NULL) || overwrite) {
-		if ((line = (char*)malloc(strlen(key) + strlen(value) + 2)) != NULL) {
-			sprintf(line, "%s=%s", key, value);
-			return putenv(line);
-		} else {
-			return -1;
-		}
-	}
-
-	return 0;
-}
-
-int unsetenv(char *key) {
-	return putenv(key);
-}
-#endif
-
-/* Wrapper for setenv
- */
 static int add_to_environment(t_fcgi_buffer *fcgi_buffer, char *key, char *value) {
 	char buffer[8];
 	int len;
@@ -80,6 +57,21 @@ static int add_to_environment(t_fcgi_buffer *fcgi_buffer, char *key, char *value
 	return 0;
 }
 
+int add_to_environment_chroot(t_session *session, t_fcgi_buffer *fcgi_buffer, char *key, char *value) {
+	size_t ofs = 0;
+	char c;
+
+	if (session->fcgi_server != NULL) {
+		if (strncmp(value, session->fcgi_server->chroot, session->fcgi_server->chroot_len) == 0) {
+			c = value[session->fcgi_server->chroot_len];
+			if ((c == '/') || (c == '\0')) {
+				ofs = session->fcgi_server->chroot_len;
+			}
+		}
+	}
+	return add_to_environment(fcgi_buffer, key, value + ofs);
+}
+
 /* Copy a headerfield to an environment setting
  */
 static int headerfield_to_environment(t_session *session, t_fcgi_buffer *fcgi_buffer, char *key, char *envir) {
@@ -95,23 +87,34 @@ static int headerfield_to_environment(t_session *session, t_fcgi_buffer *fcgi_bu
 /* Set environment variables for CGI script.
  */
 void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
-	char ip[20], len[20], value[10], sname[256];
+	char ip[20], len[20], value[10], *name;
+	size_t len1, len2;
 	unsigned char *ip_address;
 	t_keyvalue *envir;
+#ifdef HAVE_SSL
+	char subject[SSL_VAR_SIZE + 1], issuer[SSL_VAR_SIZE + 1];
+#endif
 
 	add_to_environment(fcgi_buffer, "GATEWAY_INTERFACE", "CGI/1.1");
-	add_to_environment(fcgi_buffer, "DOCUMENT_ROOT", session->host->website_root);
+
 	add_to_environment(fcgi_buffer, "REQUEST_METHOD", session->method);
 	add_to_environment(fcgi_buffer, "REQUEST_URI", session->uri);
 	if (session->uri[strlen(session->uri) - 1] == '/') {
-		strncpy(sname, session->uri, 255);
-		strncat(sname, session->host->start_file, 255);
-		sname[255] = '\0';
-		add_to_environment(fcgi_buffer, "SCRIPT_NAME", sname);
+		len1 = strlen(session->uri);
+		len2 = strlen(session->host->start_file);
+		if ((name = (char*)malloc(len1 + len2 + 1)) != NULL) {
+			memcpy(name, session->uri, len1);
+			memcpy(name + len1, session->host->start_file, len2);
+			name[len1 + len2] = '\0';
+			add_to_environment(fcgi_buffer, "SCRIPT_NAME", name);
+			free(name);
+		}
 	} else {
 		add_to_environment(fcgi_buffer, "SCRIPT_NAME", session->uri);
 	}
-	add_to_environment(fcgi_buffer, "SCRIPT_FILENAME", session->file_on_disk);
+
+	add_to_environment_chroot(session, fcgi_buffer, "SCRIPT_FILENAME", session->file_on_disk);
+	add_to_environment_chroot(session, fcgi_buffer, "DOCUMENT_ROOT", session->host->website_root);
 
 	ip_address = (unsigned char*)&(session->ip_address);
 	ip[19] = '\0';
@@ -184,4 +187,13 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 		}
 		envir = envir->next;
 	}
+
+#ifdef HAVE_SSL
+	if (session->binding->use_ssl) {
+		if (get_client_certificate(session->ssl_data, SSL_VAR_SIZE, subject, issuer) == 0) {
+			add_to_environment(fcgi_buffer, "SSL_CLIENT_SUBJECT", subject);
+			add_to_environment(fcgi_buffer, "SSL_CLIENT_ISSUER", issuer);
+		}
+	}
+#endif
 }
