@@ -46,9 +46,6 @@
 #ifdef HAVE_SSL
 #include "libssl.h"
 #endif
-#ifdef HAVE_PLUGIN
-#include "plugin.h"
-#endif
 #ifdef HAVE_CACHE
 #include "cache.h"
 #endif
@@ -66,7 +63,11 @@
 #define rs_QUIT_SERVER           1
 #define rs_UNBAN_CLIENTS         2
 #define rs_TOUCH_LOGFILES        3
-			
+
+#ifdef HAVE_SSL
+#define RANDOM_BUFFER_SIZE     512
+#endif
+
 typedef struct {
 	char *config_dir;
 	bool daemon;
@@ -88,9 +89,6 @@ char *version_string = "Hiawatha v"VERSION
 #ifdef HAVE_COMMAND
 			", CommandChannel"
 #endif
-#ifdef HAVE_PLUGIN
-			", plugin"
-#endif
 #ifdef DEBUG
 			", debug"
 #endif
@@ -139,11 +137,32 @@ void touch_logfiles(t_config *config, char *dir) {
 	}
 }
 
+/* Check if the requested file is a CGI program.
+ */
+void check_target_is_cgi(t_session *session) {
+	t_cgi_handler *cgi;
+
+	if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->file_on_disk)) != NULL) {
+		session->cgi_type = fastcgi;
+	} else if (in_charlist(file_extension(session->file_on_disk), &(session->config->cgi_extension))) {
+		session->cgi_type = program;
+	} else {
+		cgi = session->config->cgi_handler;
+		while (cgi != NULL) {
+			if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
+				session->cgi_handler = cgi->handler;
+				session->cgi_type = script;
+				break;
+			}
+			cgi = cgi->next;
+		}
+	}
+}
+
 /* Handle an HTTP error.
  */
 int handle_error(t_session *session) {
 	int retval = 200, length;
-	t_cgi_handler *cgi;
 	
 	session->handling_error = true;
 	session->mimetype = NULL;
@@ -157,23 +176,9 @@ int handle_error(t_session *session) {
 	strcpy(session->file_on_disk, session->host->website_root);
 	strcpy(session->file_on_disk + length, session->host->error_handler);
 
-	if ((session->is_cgi_script = in_charlist(file_extension(session->file_on_disk), &(session->config->cgi_extension))) == false) {
-		if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->file_on_disk)) != NULL) {
-			session->is_cgi_script = true;
-		} else {
-			cgi = session->config->cgi_handler;
-			while (cgi != NULL) {
-				if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
-					session->is_cgi_script = true;
-					session->cgi_handler = cgi->handler;
-					break;
-				}
-				cgi = cgi->next;
-			}
-		}
-	}
+	check_target_is_cgi(session);
 
-	if (session->is_cgi_script) {
+	if (session->cgi_type != none) {
 		if (session->host->return_code != -1) {
 			session->return_code = session->host->return_code;
 		}
@@ -189,26 +194,11 @@ int handle_error(t_session *session) {
  */
 int handle_request(t_session *session) {
 	int retval = 200;
-	t_cgi_handler *cgi;
 
-	if ((session->is_cgi_script = in_charlist(file_extension(session->file_on_disk), &(session->config->cgi_extension))) == false) {
-		if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->file_on_disk)) != NULL) {
-			session->is_cgi_script = true;
-		} else {
-			cgi = session->config->cgi_handler;
-			while (cgi != NULL) {
-				if (in_charlist(file_extension(session->file_on_disk), &(cgi->extension))) {
-					session->cgi_handler = cgi->handler;
-					session->is_cgi_script = true;
-					break;
-				}
-				cgi = cgi->next;
-			}
-		}
-	}
+	check_target_is_cgi(session);
 
 	if ((strcmp(session->method, "GET") == 0) || session->head_request) {
-		if (session->is_cgi_script) {
+		if (session->cgi_type != none) {
 			session->body = NULL;
 			retval = execute_cgi(session);
 		} else {
@@ -220,7 +210,7 @@ int handle_request(t_session *session) {
 			}
 		}
 	} else if (strcmp(session->method, "POST") == 0) {
-		if (session->is_cgi_script) {
+		if (session->cgi_type != none) {
 			retval = execute_cgi(session);
 		} else {
 			retval = 405;
@@ -563,23 +553,9 @@ int uri_to_path(t_session *session) {
 	return 200;
 }
 
-#ifdef HAVE_PLUGIN
-/* Should the plugin be used?
- */
-bool is_plugin_uri(t_session *session) {
-	if ((session->config->plugin_uri != NULL) && (session->host->plugin_active)) {
-		if (strcmp(session->uri, session->config->plugin_uri) == 0) {
-			return true;
-		}
-	}
-
-	return false;
-}
-#endif
-
 t_access allow_client(t_session *session) {
 	char *x_forwarded_for;
-	unsigned long forwarded_ip;
+	t_ipv4 forwarded_ip;
 	t_access access;
 
 	if ((access = ip_allowed(session->ip_address, session->host->access_list)) != allow) {
@@ -677,26 +653,6 @@ int serve_client(t_session *session) {
 		return 403;
 	}
 
-#ifdef HAVE_PLUGIN
-	if (is_plugin_uri(session)) {
-		switch (allow_client(session)) {
-			case deny:
-				log_error(session, fb_accesslist);
-				return 403;
-				break;
-			case allow:
-				break;
-			case pwd:
-			case unknown:
-				if (http_authentication_oke(session) == false) {
-					return 401;
-				}
-				break;
-		}
-		return handle_plugin(session);
-	}
-#endif
-
 	if ((result = copy_directory_settings(session)) != 200) {
 		return result;
 	}
@@ -817,12 +773,16 @@ void handle_request_result(t_session *session, int result) {
 		case 304:
 		case 412:
 		case 413:
-			send_header(session, result);
-			send_buffer(session, "\r\n", 2);
+			if (session->data_sent == false) {
+				send_header(session, result);
+				send_buffer(session, "\r\n", 2);
+			}
 			break;
 		case 400:
 			log_garbage(session);
-			send_code(session, result);
+			if (session->data_sent == false) {
+				send_code(session, result);
+			}
 			if ((session->config->ban_on_garbage > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_garbage, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending garbage");
@@ -831,21 +791,23 @@ void handle_request_result(t_session *session, int result) {
 		case 500:
 			session->keep_alive = false;
 		default:
-			if ((session->host->error_handler == NULL) || (result == 301)) {
-				if (send_code(session, result) == -1) {
-					session->keep_alive = false;
-				}
-			} else {
-				session->error_code = result;
-				switch (handle_error(session)) {
-					case -1:
+			if (session->data_sent == false) {
+				if ((session->host->error_handler == NULL) || (result == 301)) {
+					if (send_code(session, result) == -1) {
 						session->keep_alive = false;
-					case 200:
-						break;
-					default:
-						if (send_code(session, result) == -1) {
+					}
+				} else {
+					session->error_code = result;
+					switch (handle_error(session)) {
+						case -1:
 							session->keep_alive = false;
-						}
+						case 200:
+							break;
+						default:
+							if (send_code(session, result) == -1) {
+								session->keep_alive = false;
+							}
+					}
 				}
 			}
 	}
@@ -1014,8 +976,8 @@ int accept_connection(t_binding *binding, t_config *config) {
 	session->binding = binding;
 	init_session(session);
 
-	memset((void*)&caddr, 0, sizeof(struct sockaddr_in));
 	size = sizeof(struct sockaddr_in);
+	memset((void*)&caddr, 0, size);
 	if ((session->client_socket = accept(binding->socket, (struct sockaddr*)&caddr, &size)) == -1) {
 		free(session);
 		if (errno != EINTR) {
@@ -1085,7 +1047,13 @@ int run_server(t_settings *settings) {
 #ifdef HAVE_COMMAND
 	t_admin            *admin;
 	struct sockaddr_in caddr;
-	int                size, admin_socket;
+	socklen_t          size;
+	int                admin_socket;
+#endif
+#ifdef HAVE_SSL
+#ifndef HAVE_DEV_URANDOM
+	char               random_buffer[RANDOM_BUFFER_SIZE];
+#endif
 #endif
 	pid_t              pid;
 	t_binding          *binding;
@@ -1134,7 +1102,12 @@ int run_server(t_settings *settings) {
 	}
 
 #ifdef HAVE_SSL
-	ssl_init();
+#ifdef HAVE_DEV_URANDOM
+	ssl_init(NULL, 0);
+#else
+	fill_random_buffer(config, random_buffer, RANDOM_BUFFER_SIZE);
+	ssl_init(random_buffer, RANDOM_BUFFER_SIZE);
+#endif
 	binding = config->binding;
 	while (binding != NULL) {
 		if (binding->use_ssl) {
@@ -1154,19 +1127,6 @@ int run_server(t_settings *settings) {
 	}
 #endif
 
-#ifdef HAVE_PLUGIN
-	switch (plugin_init()) {
-		case 0:
-			if (config->plugin_uri == NULL) {
-				config->plugin_uri = get_plugin_uri();
-			}
-			break;
-		default:
-			fprintf(stderr, "Error while initializing plugin.\n");
-			return EXIT_FAILURE;
-	}
-#endif
-
 	touch_logfiles(config, config->server_root);
 	tzset();
 	clearenv();
@@ -1294,9 +1254,6 @@ int run_server(t_settings *settings) {
 #endif
 #ifdef HAVE_COMMAND
 				check_adminlist();
-#endif
-#ifdef HAVE_PLUGIN
-				plugin_timer();
 #endif
 				switch (received_signal) {
 					case rs_NONE:
@@ -1351,6 +1308,7 @@ int run_server(t_settings *settings) {
 				while (binding != NULL) {
 					if (FD_ISSET(binding->socket, &read_fds)) {
 						size = sizeof(struct sockaddr_in);
+						memset((void*)&caddr, 0, size);
 						if ((admin_socket = accept(binding->socket, (struct sockaddr*)&caddr, &size)) == -1) {
 							if (errno != EINTR) {
 								log_string(config->system_logfile, "Fatal error accepting CommandChannel connection");
@@ -1397,10 +1355,6 @@ int run_server(t_settings *settings) {
 
 	log_string(config->system_logfile, "Hiawatha v"VERSION" stopped");
 
-#ifdef HAVE_PLUGIN
-	plugin_destroy();
-#endif
-
 	return EXIT_SUCCESS;
 }
 
