@@ -315,25 +315,21 @@ int if_modified_since(int handle, char *datestr) {
 	struct tm *fdate, rdate;
 	time_t file_date, req_date;
 
-	do {
-		if (fstat(handle, &status) == -1) {
-			break;
-		} else if ((fdate = gmtime(&(status.st_mtime))) == NULL) {
-			break;
-		} else if ((file_date = mktime(fdate)) == -1) {
-			break;
-		} else if (parse_datestr(datestr, &rdate) == -1) {
-			break;
-		} else if ((req_date = mktime(&rdate)) == -1) {
-			break;
-		} else if (file_date > req_date) {
-			return 1;
-		} else {
-			return 0;
-		}
-	} while (false);
+	if (fstat(handle, &status) == -1) {
+		return -1;
+	} else if ((fdate = gmtime(&(status.st_mtime))) == NULL) {
+		return -1;
+	} else if ((file_date = mktime(fdate)) == -1) {
+		return -1;
+	} else if (parse_datestr(datestr, &rdate) == -1) {
+		return -1;
+	} else if ((req_date = mktime(&rdate)) == -1) {
+		return -1;
+	} else if (file_date > req_date) {
+		return 1;
+	}
 
-	return -1;
+	return 0;
 }
 
 /* Open a file (searches in directory where file 'neighbour' is located if not found).
