@@ -13,7 +13,7 @@ void sfree(void* ptr) {
 
 /* Convert an IP address to an unsigned long.
  */
-int parse_ip(char *line, unsigned long *l_ip) {
+int parse_ip(char *line, t_ipv4 *l_ip) {
 	char *ip, *c, org;
 	int  i, byte;
 
@@ -205,7 +205,7 @@ t_accesslist *remove_accesslist(t_accesslist *list) {
 /* Return the access status of an IP address.
  */
 #define masked(ip, mask) ((ip << mask) >> mask)
-t_access ip_allowed(unsigned long ip, t_accesslist *list) {
+t_access ip_allowed(t_ipv4 ip, t_accesslist *list) {
 	while (list != NULL) {
 		if (list->netmask == 32) {
 			return list->access;
