@@ -2,6 +2,7 @@
 #define _LIBLIST_H
 
 #include <stdbool.h>
+#include "global.h"
 
 typedef enum { deny, allow, pwd, unknown } t_access;
 
@@ -11,7 +12,7 @@ typedef struct type_charlist {
 } t_charlist;
 
 typedef struct type_accesslist {
-	unsigned long ip;
+	t_ipv4 ip;
 	int netmask;
 	t_access access;
 
@@ -38,10 +39,10 @@ void init_charlist(t_charlist *list);
 int parse_charlist(char *value, t_charlist *list);
 bool in_charlist(char *item, t_charlist *list);
 
-int parse_ip(char *line, unsigned long *l_ip);
+int parse_ip(char *line, t_ipv4 *l_ip);
 t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list);
 t_accesslist *remove_accesslist(t_accesslist *list);
-t_access ip_allowed(unsigned long ip, t_accesslist *list);
+t_access ip_allowed(t_ipv4 ip, t_accesslist *list);
 
 int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator);
 t_keyvalue *remove_keyvaluelist(t_keyvalue *list);
