@@ -272,11 +272,12 @@ int prevent_XSS(char *line) {
 
 /* Return an errormessage.
  */
-const char *errorstr(int code) {
+const char *http_error(int code) {
+	int i;
 	static const struct {
 		int code;
-		const char *msg;
-	} translation[] = {
+		const char *message;
+	} error[] = {
 /*		{100, "Continue"},
 		{101, "Switching Protocols"},
 		{102, "Processing"},
@@ -347,15 +348,14 @@ const char *errorstr(int code) {
 		{505, "HTTP Version Not Supported"},
 		{0  , "Unknown Error"}
 	};
-	int i;
 
-	for (i=0; translation[i].code!=0; i++) {
-		if (translation[i].code == code) {
-			return translation[i].msg;
+	for (i=0; error[i].code != 0; i++) {
+		if (error[i].code == code) {
+			return error[i].message;
 		}
 	}
 
-	return "unknown error code";
+	return NULL;
 }
 
 /* Decode an base64 encoded string.
