@@ -16,7 +16,7 @@ int parse_special_chars(char *line, int length);
 int prevent_CMDi(char *line);
 int prevent_SQLi(char *line, int length, char **newline);
 int prevent_XSS(char *line);
-const char *errorstr(int code);
+const char *http_error(int code);
 bool decode_base64(char *base64);
 char *file_extension(char *filename);
 char *str_replace(char *line, char *from, char *to);
