@@ -36,7 +36,7 @@ static void print_timestamp(char *str) {
 
 /* Write an IP address to a logfile.
  */
-static void print_ip_address(char *str, long ip_address) {
+static void print_ip_address(char *str, t_ipv4 ip_address) {
 	unsigned char *ip;
 
 	ip = (unsigned char*)&ip_address;
@@ -230,7 +230,7 @@ void log_garbage(t_session *session) {
 
 /* Log an unbanning.
  */
-void log_unban(char *logfile, unsigned long ip_address, unsigned long connect_attempts) {
+void log_unban(char *logfile, t_ipv4 ip_address, unsigned long connect_attempts) {
 	FILE *fp;
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
@@ -246,20 +246,30 @@ void log_unban(char *logfile, unsigned long ip_address, unsigned long connect_at
  */
 void log_file_error(t_session *session, char *file, char *mesg) {
 	FILE *fp;
-	char *c, str[TIMESTAMP_SIZE];
+	char *c, str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
+	int len = 0;
 
 	c = mesg;
 	while (*c != '\0') {
 		if (*c == '\n') {
-			*c = '|';
+			if (*(c + 1) == '\0') {
+				*c = '\0';
+			} else {
+				*c = '|';
+			}
+		} else {
+			len++;
 		}
 		c++;
 	}
-
-	if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
-		print_timestamp(str);
-		fprintf(fp, "%s%s|%s\n", str, file, secure_string(mesg));
-		fclose(fp);
+	
+	if (len > 0) {
+		if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
+			print_timestamp(str);
+			print_ip_address(str + strlen(str), session->ip_address);
+			fprintf(fp, "%s%s|%s\n", str, file, secure_string(mesg));
+			fclose(fp);
+		}
 	}
 }
 
