@@ -39,6 +39,8 @@ char *hs_range  = "Accept-Ranges: bytes\r\n";   // 22
 char *hs_gzip   = "Content-Encoding: gzip\r\n"; // 24
 char *hs_eol    = "\r\n";                       //  2
 
+char *unknown_code = "unknown error code";
+
 /* Send a char buffer to the client. Traffic throttling is handled here.
  */
 static int send_to_client(t_session *session, const char *buffer, int size) {
@@ -147,7 +149,6 @@ int send_buffer(t_session *session, const char *buffer, int size) {
 /* Send a HTTP header to the client. Header is not closed by this function.
  */
 int send_header(t_session *session, int code) {
-	int retval = -1;
 	char ecode[5], timestr[64];
 	const char *emesg;
 	time_t t;
@@ -156,81 +157,82 @@ int send_header(t_session *session, int code) {
 	time(&t);
 	s = gmtime(&t);
 	strftime(timestr, 64, "%a, %d %b %Y %X GMT\r\n", s);
-
+	
 	/* Send HTTP header.
 	 */
 	ecode[4] = '\0';
 	snprintf(ecode, 4, "%d", code);
-	emesg = errorstr(code);
-	do {
-		if (session->http_version != NULL) {
-			if (*(session->http_version + 7) == '0') {
-				if (send_buffer(session, hs_http10, 9) == -1) {
-					break;
-				}
-			} else {
-				if (send_buffer(session, hs_http11, 9) == -1) {
-					break;
-				}
+	if ((emesg = http_error(code)) == NULL) {
+		emesg = unknown_code;
+	}
+	
+	session->data_sent = true;
+
+	if (session->http_version != NULL) {
+		if (*(session->http_version + 7) == '0') {
+			if (send_buffer(session, hs_http10, 9) == -1) {
+				return -1;
 			}
 		} else {
 			if (send_buffer(session, hs_http11, 9) == -1) {
-				break;
+				return -1;
 			}
 		}
-		
-		if (send_buffer(session, ecode, 3) == -1) {
-			break;
-		} else if (send_buffer(session, " ", 1) == -1) {
-			break;
-		} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
-			break;
-		} else if (send_buffer(session, hs_eol, 2) == -1) {
-			break;
-		} else if (send_buffer(session, "Date: ", 6) == -1) {
-			break;
-		} else if (send_buffer(session, timestr, strlen(timestr)) == -1) {
-			break;
-		} else if (send_buffer(session, hs_server, 8) == -1) {
-			break;
-		} else if (send_buffer(session, session->config->server_string, strlen(session->config->server_string)) == -1) {
-			break;
-		} else if (send_buffer(session, hs_eol, 2) == -1) {
-			break;
-		} else if ((session->is_cgi_script == false) && (session->uri_is_dir == false)) {
-			if (send_buffer(session, hs_range, 22) == -1) {
-				break;
-			}
-		}
-		if (send_buffer(session, hs_conn, 12) == -1) {
-			break;
-		} else if (session->keep_alive) {
-			if (send_buffer(session, hs_conka, 12) == -1) {
-				break;
-			}
-		} else if (send_buffer(session, hs_concl, 7) == -1) {
-			break;
-		}
-		if (session->encode_gzip) {
-			if (send_buffer(session, hs_gzip, 24) == -1) {
-				break;
-			}
+	} else {
+		if (send_buffer(session, hs_http11, 9) == -1) {
+			return -1;
 		}
+	}
 
-		if (session->mimetype != NULL) {
-			if (send_buffer(session, hs_contyp, 14) == -1) {
-				break;
-			} else if (send_buffer(session, session->mimetype, strlen(session->mimetype)) == -1) {
-				break;
-			} else if (send_buffer(session, "\r\n", 2) == -1) {
-				break;
-			}
+	if (send_buffer(session, ecode, 3) == -1) {
+		return -1;
+	} else if (send_buffer(session, " ", 1) == -1) {
+		return -1;
+	} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
+		return -1;
+	} else if (send_buffer(session, hs_eol, 2) == -1) {
+		return -1;
+	} else if (send_buffer(session, "Date: ", 6) == -1) {
+		return -1;
+	} else if (send_buffer(session, timestr, strlen(timestr)) == -1) {
+		return -1;
+	} else if (send_buffer(session, hs_server, 8) == -1) {
+		return -1;
+	} else if (send_buffer(session, session->config->server_string, strlen(session->config->server_string)) == -1) {
+		return -1;
+	} else if (send_buffer(session, hs_eol, 2) == -1) {
+		return -1;
+	} else if ((session->cgi_type == none) && (session->uri_is_dir == false)) {
+		if (send_buffer(session, hs_range, 22) == -1) {
+			return -1;
 		}
+	}
+	if (send_buffer(session, hs_conn, 12) == -1) {
+		return -1;
+	} else if (session->keep_alive) {
+		if (send_buffer(session, hs_conka, 12) == -1) {
+			return -1;
+		}
+	} else if (send_buffer(session, hs_concl, 7) == -1) {
+		return -1;
+	}
+	if (session->encode_gzip) {
+		if (send_buffer(session, hs_gzip, 24) == -1) {
+			return -1;
+		}
+	}
 
-		retval = 0;
-	} while (false);
+	if (session->mimetype != NULL) {
+		if (send_buffer(session, hs_contyp, 14) == -1) {
+			return -1;
+		} else if (send_buffer(session, session->mimetype, strlen(session->mimetype)) == -1) {
+			return -1;
+		} else if (send_buffer(session, "\r\n", 2) == -1) {
+			return -1;
+		}
+	}
 
-	return retval;
+	return 0;
 }
 
 /* Send a datachunk to the client, used by run_script() in target.c
@@ -301,7 +303,7 @@ int send_chunk(t_session *session, const char *chunk, int size) {
 /* Send a HTTP code to the client. Used in case of an error.
  */
 int send_code(t_session *session, int code) {
-	int retval = -1, default_port;
+	int default_port;
 	char ecode[5], len[10], port[10];
 	const char *emesg;
 
@@ -313,27 +315,27 @@ int send_code(t_session *session, int code) {
 	 */
 	session->mimetype = NULL;
 	if (send_header(session, code) == -1) {
-		return retval;
+		return -1;
 	}
 
 	switch (code) {
 		case 301:
 			if (send_buffer(session, hs_lctn, 14) == -1) {
-				break;
+				return -1;
 			}
 #ifdef HAVE_SSL
 			if (session->binding->use_ssl || (session->cause_of_301 == require_ssl)) {
 				if (send_buffer(session, "s", 1) == -1) {
-					break;
+					return -1;
 				}
 			}
 #endif
 			if (send_buffer(session, hs_slash, 3) == -1) {
-				break;
+				return -1;
 			}
 
 			if (send_buffer(session, *(session->host->hostname.item), strlen(*(session->host->hostname.item))) == -1) {
-				break;
+				return -1;
 			}
 
 #ifdef HAVE_SSL
@@ -347,30 +349,29 @@ int send_code(t_session *session, int code) {
 				port[9] = '\0';
 				snprintf(port, 9, ":%d", session->binding->port);
 				if (send_buffer(session, port, strlen(port)) == -1) {
-					break;
+					return -1;
 				}
 			}
 			if (send_buffer(session, session->uri, strlen(session->uri)) == -1) {
-				break;
+				return -1;
 			}
 #ifdef HAVE_SSL
 			if (session->cause_of_301 == missing_slash)
 #endif
 				if (send_buffer(session, "/", 1) == -1) {
-					break;
+					return -1;
 				}
 
 			if (session->vars != NULL) {
 				if (send_buffer(session, "?", 1) == -1) {
-					break;
+					return -1;
 				} else if (send_buffer(session, session->vars, strlen(session->vars)) == -1) {
-					break;
+					return -1;
 				}
 			}
 			if (send_buffer(session, "\r\n", 2) == -1) {
-				break;
+				return -1;
 			}
-			retval = 0;
 			break;
 		case 401:
 			if (session->host->auth_method == basic) {
@@ -378,66 +379,57 @@ int send_code(t_session *session, int code) {
 			} else {
 				send_digest_auth(session);
 			}
-			retval = 0;
-			break;
-		default:
-			retval = 0;
 			break;
 	}
 
-	if (retval == 0) {
-		do {
-			retval = -1;
-			ecode[4] = '\0';
-			snprintf(ecode, 4, "%d", code);
-			emesg = errorstr(code);
-			len[9] = '\0';
-			snprintf(len, 9, "%d", 2 * strlen(emesg) + 291);
+	ecode[4] = '\0';
+	snprintf(ecode, 4, "%d", code);
 
-			if (send_buffer(session, "Content-Length: ", 16) == -1) {
-				break;
-			} else if (send_buffer(session, len, strlen(len)) == -1) {
-				break;
-			} else if (send_buffer(session, "\r\n", 2) == -1) {
-				break;
-			} else if (send_buffer(session, hs_contyp, 14) == -1) {
-				break;
-			} else if (send_buffer(session, "text/html\r\n\r\n", 13) == -1) {
-				break;
-			} else if (session->head_request) {
-				retval = 0;
-				break;
-			} else if (send_buffer(session, "<html><head><title>", 19) == -1) {
-				break;
-			} else if (send_buffer(session, ecode, 3) == -1) {
-				break;
-			} else if (send_buffer(session, " - ", 3) == -1) {
-				break;
-			} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
-				break;
-			} else if (send_buffer(session,	"</title><style type=\"text/css\">\n" // 32
-											"<!--\n" // 5
-											"BODY { color:#ffffff ; background-color:#00000a }\n" // 50
-											"DIV { font-family:sans-serif ; font-size:30px ; letter-spacing:20px ; " // 70
-											"text-align:center ; position:relative ; top:250px }\n" // 52
-											"--></style></head>\n" // 19
-											"<body><div>", 239) == -1) { // 11
-				break;
-			} else if (send_buffer(session, ecode, 3) == -1) {
-				break;
-			} else if (send_buffer(session, " - ", 3) == -1) {
-				break;
-			} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
-				break;
-			} else if (send_buffer(session, "</div></body></html>\n", 21) == -1) {
-				break;
-			}
+	if ((emesg = http_error(code)) == NULL) {
+		emesg = unknown_code;
+	}
+	len[9] = '\0';
+	snprintf(len, 9, "%d", 2 * strlen(emesg) + 291);
 
-			retval = 0;
-		} while (false);
+	if (send_buffer(session, "Content-Length: ", 16) == -1) {
+		return -1;
+	} else if (send_buffer(session, len, strlen(len)) == -1) {
+		return -1;
+	} else if (send_buffer(session, "\r\n", 2) == -1) {
+		return -1;
+	} else if (send_buffer(session, hs_contyp, 14) == -1) {
+		return -1;
+	} else if (send_buffer(session, "text/html\r\n\r\n", 13) == -1) {
+		return -1;
+	} else if (session->head_request) {
+		return 0;
+	} else if (send_buffer(session, "<html><head><title>", 19) == -1) {
+		return -1;
+	} else if (send_buffer(session, ecode, 3) == -1) {
+		return -1;
+	} else if (send_buffer(session, " - ", 3) == -1) {
+		return -1;
+	} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
+		return -1;
+	} else if (send_buffer(session,	"</title><style type=\"text/css\">\n" // 32
+									"<!--\n" // 5
+									"BODY { color:#ffffff ; background-color:#00000a }\n" // 50
+									"DIV { font-family:sans-serif ; font-size:30px ; letter-spacing:20px ; " // 70
+									"text-align:center ; position:relative ; top:250px }\n" // 52
+									"--></style></head>\n" // 19
+									"<body><div>", 239) == -1) { // 11
+		return -1;
+	} else if (send_buffer(session, ecode, 3) == -1) {
+		return -1;
+	} else if (send_buffer(session, " - ", 3) == -1) {
+		return -1;
+	} else if (send_buffer(session, emesg, strlen(emesg)) == -1) {
+		return -1;
+	} else if (send_buffer(session, "</div></body></html>\n", 21) == -1) {
+		return -1;
 	}
 
-	return retval;
+	return 0;
 }
 
 /* Send directly to buffer, unbuffered
@@ -565,11 +557,11 @@ void send_basic_auth(t_session *session) {
 /* Send a Digest Authentication message to the client.
  */
 void send_digest_auth(t_session *session) {
-	char nonce[2 * NONCE_DIGITS];
+	char nonce[2 * NONCE_DIGITS + 1];
 	int i;
 
 	for (i = 0; i < NONCE_DIGITS; i++) {
-		sprintf(nonce + (2 * i), "%02X", (char)(random() & 255));
+		snprintf(nonce + (2 * i), 3, "%02hhX", (char)random());
 	}
 	
 	if (send_buffer(session, "WWW-Authenticate: Digest", 24) == -1) {
