@@ -8,6 +8,9 @@
 #ifdef HAVE_COMMAND
 #include <netinet/in.h>
 #endif
+#ifdef HAVE_SSL
+#include <time.h>
+#endif
 #include "serverconfig.h"
 #include "libstr.h"
 #include "libfs.h"
@@ -46,9 +49,6 @@ static t_host *new_host(void) {
 		host->use_gz_file        = false;
 		host->require_resolve_ip = false;
 		host->access_list        = NULL;
-#ifdef HAVE_PLUGIN
-		host->plugin_active      = false;
-#endif
 		host->login_message      = "Private page";
 		host->passwordfile       = NULL;
 		host->groupfile          = NULL;
@@ -196,9 +196,6 @@ t_config *default_config(void) {
 		config->time_for_cgi       = 5;
 		config->kill_timedout_cgi  = true;
 		config->first_host         = new_host();
-#ifdef HAVE_PLUGIN
-		config->plugin_uri         = NULL;
-#endif
 		config->mimetype           = NULL;
 		config->directory          = NULL;
 		config->throttle           = NULL;
@@ -206,6 +203,8 @@ t_config *default_config(void) {
 		config->cgi_wrapper        = NULL;
 		config->wrap_user_cgi      = false;
 		config->log_format         = hiawatha;
+		config->user_directory     = "public_html";
+		config->user_directory_set = false;
 
 		config->pidfile            = PIDFILE_DIR"/hiawatha.pid";
 		config->system_logfile     = LOG_DIR"/system.log";
@@ -649,14 +648,6 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				return true;
 			}
 		}
-#ifdef HAVE_PLUGIN
-	} else if (strcmp(key, "pluginuri") == 0) {
-		if (strlen(value) < 16) {
-			if ((config->plugin_uri = strdup(value)) != NULL) {
-				return true;
-			}
-		}
-#endif
 	} else if (strcmp(key, "reconnectdelay") == 0) {
 		if ((config->reconnect_delay = str2int(value)) != -1) {
 			return true;
@@ -720,6 +711,13 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		if ((config->time_for_cgi = str2int(value)) >= 1) {
 			return true;
 		}
+	} else if (strcmp(key, "userdirectory") == 0) {
+		if ((*value != '/') && (strchr(value, '.') == NULL)) {
+			if ((config->user_directory = strdup(value)) != NULL) {
+				config->user_directory_set = true;
+				return true;
+			}
+		}
 	} else if (strcmp(key, "wrapusercgi") == 0) {
 		if (parse_yesno(value, &(config->wrap_user_cgi)) == 0) {
 			return true;
@@ -868,12 +866,6 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if ((host->index_style = strdup(value)) != NULL) {
 			return true;
 		}
-#ifdef HAVE_PLUGIN
-	} else if (strcmp(key, "pluginactive") == 0) {
-		if (parse_yesno(value, &(host->plugin_active)) == 0) {
-			return true;
-		}
-#endif
 	} else if (strcmp(key, "preventcmdi") == 0) {
 		if (parse_yesno(value, &(host->prevent_cmdi)) == 0) {
 			return true;
@@ -1564,3 +1556,73 @@ unsigned short get_throttlespeed(char *type, t_throttle *throttle) {
 
 	return speed;
 }
+
+#ifdef HAVE_SSL
+#ifndef HAVE_DEV_URANDOM
+static int mod_memadd(char *src, int src_size, char *dest, int dest_size, int dest_ofs) {
+	int src_ofs = 0;
+
+	while (src_ofs < src_size) {
+		dest[dest_ofs++] ^= src[src_ofs++];
+		if (dest_ofs >= dest_size) {
+			dest_ofs = 0;
+		}
+	}
+
+	return dest_ofs;
+}
+
+void fill_random_buffer(t_config *config, char *buffer, int size) {
+	int i, pos = 0;
+	time_t now;
+	t_host *host;
+	t_directory *dir;
+	t_binding *binding;
+	t_cgi_handler *cgi;
+	t_mimename *mime;
+
+	now = time(NULL);
+	for (i = 0; i < (size / sizeof(time_t)); i++) {
+		pos = mod_memadd((char*)&now, sizeof(time_t), buffer, size, pos);
+	}
+
+	pos = mod_memadd((char*)config, sizeof(t_config), buffer, size, pos);
+
+	host = config->first_host;
+	while (host != NULL) {
+		pos = mod_memadd((char*)host, sizeof(t_host), buffer, size, pos);
+		for (i = 0; i < host->hostname.size; i++) {
+			pos = mod_memadd(*(host->hostname.item + i), strlen(*(host->hostname.item + i)), buffer, size, pos);
+		}
+		pos = mod_memadd(host->website_root, strlen(host->website_root), buffer, size, pos);
+		host = host->next;
+	}
+
+	dir = config->directory;
+	while (dir != NULL) {
+		pos = mod_memadd((char*)dir, sizeof(t_directory), buffer, size, pos);
+		pos = mod_memadd(dir->path, strlen(dir->path), buffer, size, pos);
+		dir = dir->next;
+	}
+
+	binding = config->binding;
+	while (binding != NULL) {
+		pos = mod_memadd((char*)binding, sizeof(t_binding), buffer, size, pos);
+		binding = binding->next;
+	}
+
+	cgi = config->cgi_handler;
+	while (cgi != NULL) {
+		pos = mod_memadd((char*)cgi, sizeof(t_cgi_handler), buffer, size, pos);
+		pos = mod_memadd(cgi->handler, strlen(cgi->handler), buffer, size, pos);
+		cgi = cgi->next;
+	}
+
+	mime = config->mimetype->mimename;
+	while (mime != NULL) {
+		pos = mod_memadd(mime->name, strlen(mime->name), buffer, size, pos);
+		mime = mime->next;
+	}
+}
+#endif
+#endif
