@@ -4,6 +4,7 @@
 #include "config.h"
 #include <stdbool.h>
 #include <pthread.h>
+#include "global.h"
 #include "mimetype.h"
 #include "libfs.h"
 #include "liblist.h"
@@ -31,7 +32,7 @@ typedef struct type_connect_to {
 } t_connect_to;
 
 typedef struct type_cgi_session {
-	unsigned long client_ip;
+	t_ipv4 client_ip;
 	t_connect_to  *connect_to;
 	int timer;
 
@@ -61,7 +62,7 @@ typedef struct type_throttle {
 
 typedef struct type_binding {
 	int           port;
-	unsigned long interface;
+	t_ipv4        interface;
 	char          *binding_id;
 #ifdef HAVE_SSL
 	bool          use_ssl;
@@ -131,9 +132,6 @@ typedef struct type_host {
 	bool          show_index;
 	bool          use_gz_file;
 	bool          require_resolve_ip;
-#ifdef HAVE_PLUGIN 
-	bool          plugin_active;
-#endif
 	char          *login_message;
 	char          *passwordfile;
 	t_auth_method auth_method;
@@ -189,11 +187,9 @@ typedef struct type_config {
 	char          *garbage_logfile;
 	char          *pidfile;
 	t_accesslist  *logfile_mask;
+	char          *user_directory;
+	bool          user_directory_set;
 	
-#ifdef HAVE_PLUGIN
-	char          *plugin_uri;
-#endif
-
 	t_mimetype    *mimetype;
 	t_host        *first_host;
 	t_directory   *directory;
@@ -236,4 +232,10 @@ t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding);
 int read_throttleconfig(char *configfile, t_throttle **throttle);
 unsigned short get_throttlespeed(char *type, t_throttle *throttle);
 
+#ifdef HAVE_SSL
+#ifndef HAVE_DEV_URANDOM
+void fill_random_buffer(t_config *config, char *buffer, int size);
+#endif
+#endif
+
 #endif
