@@ -23,7 +23,7 @@ int new_client_id = 0;
  */
 static void clear_session(t_session *session) {
 	session->head_request = false;
-	session->is_cgi_script = false;
+	session->cgi_type = none;
 	session->cgi_handler = NULL;
 	session->fcgi_server = NULL;
 	session->method = NULL;
@@ -37,6 +37,7 @@ static void clear_session(t_session *session) {
 	session->headerfields = NULL;
 	session->body = NULL;
 	session->local_user = NULL;
+	session->data_sent = false;
 #ifdef HAVE_SSL
 	session->cause_of_301 = missing_slash;
 #endif
@@ -220,12 +221,13 @@ int get_homedir(t_session *session, char *username) {
 
 	if ((pwd = getpwnam(username)) != NULL) {
 		len = strlen(pwd->pw_dir);
-		if ((session->host->website_root = (char*)malloc(len + 13)) == NULL) {
+		if ((session->host->website_root = (char*)malloc(len + strlen(session->config->user_directory) + 2)) == NULL) {
 			return -1;
 		}
 
 		memcpy(session->host->website_root, pwd->pw_dir, len);
-		memcpy(session->host->website_root + len, "/public_html\0", 13);
+		*(session->host->website_root + len) = '/';
+		strcpy(session->host->website_root + len + 1, session->config->user_directory);
 		session->websiteroot_copied = true;
 
 		return 200;
