@@ -22,6 +22,7 @@
 #ifdef HAVE_SENDFILE
 #include <sys/sendfile.h>
 #endif
+#include "global.h"
 #include "alternative.h"
 #include "libstr.h"
 #include "libfs.h"
@@ -35,9 +36,6 @@
 #ifdef HAVE_COMMAND
 #include "command.h"
 #endif
-#ifdef HAVE_PLUGIN
-#include "plugin.h"
-#endif
 
 #define MAX_VOLATILE_SIZE    1048576
 #define FILE_BUFFER_SIZE       32768
@@ -48,10 +46,6 @@
 #define FILESIZE_BUFFER_SIZE      30
 #define CGI_BUFFER_SIZE        32768
 
-#define KILOBYTE        1024
-#define MEGABYTE     1048576
-#define GIGABYTE  1073741824
-
 #define rs_QUIT       -1
 #define rs_DISCONNECT -2
 #define rs_FORCE_QUIT -3
@@ -421,7 +415,7 @@ int send_file(t_session *session) {
 int execute_cgi(t_session *session) {
 	int retval = 200, result, handle;
 	char *line, *new_line, *contyp_start, *contyp_end, *code;
-	bool in_body = false, send_in_chunks = true, wrap_cgi, use_fast_cgi;
+	bool in_body = false, send_in_chunks = true, wrap_cgi;
 	t_connect_to *connect_to;
 	t_cgi_info cgi_info;
 	pid_t cgi_pid = -1;
@@ -430,11 +424,11 @@ int execute_cgi(t_session *session) {
 	increment_counter(COUNTER_CGI);
 #endif
 
-	if ((use_fast_cgi = (session->fcgi_server != NULL))) {
-		wrap_cgi = false;
-	} else {
+	if (session->cgi_type != fastcgi) {
 		wrap_cgi = (session->config->cgi_wrapper != NULL) && ((session->host->wrap_cgi != NULL) ||
-			((session->local_user != NULL) && session->config->wrap_user_cgi));
+		           ((session->local_user != NULL) && session->config->wrap_user_cgi));
+	} else {
+		wrap_cgi = false;
 	}
 
 	/* HTTP/1.0 does not support chunked Transfer-Encoding.
@@ -443,7 +437,7 @@ int execute_cgi(t_session *session) {
 		session->keep_alive = false;
 	}
 
-	if ((wrap_cgi == false) && (use_fast_cgi == false)) {
+	if ((wrap_cgi == false) && (session->cgi_type != fastcgi)) {
 		if ((handle = open(session->file_on_disk, O_RDONLY)) == -1) {
 			if (errno == EACCES) {
 				log_error(session, fb_filesystem);
@@ -460,8 +454,8 @@ int execute_cgi(t_session *session) {
 		return 403;
 	}
 
-	if ((wrap_cgi == false) && (use_fast_cgi == false)) {
-		if (session->cgi_handler == NULL) {
+	if ((wrap_cgi == false) && (session->cgi_type != fastcgi)) {
+		if (session->cgi_type == program) {
 			switch (can_execute(session->file_on_disk, session->config->server_uid, session->config->server_gid, &(session->config->groups))) {
 				case error:
 					log_error(session, "error during CGI preprocess");
@@ -572,7 +566,7 @@ int execute_cgi(t_session *session) {
 	cgi_info.input_buffer_size = cgi_info.error_buffer_size = CGI_BUFFER_SIZE;
 	cgi_info.input_len = cgi_info.error_len = 0;
 
-	if (use_fast_cgi) {
+	if (session->cgi_type == fastcgi) {
 		cgi_info.read_header = true;
 		if ((connect_to = select_connect_to(session->fcgi_server, session->ip_address)) == NULL) {
 			return 503;
@@ -603,7 +597,7 @@ int execute_cgi(t_session *session) {
 	}
 
 	if (retval != 200) {
-		if (use_fast_cgi) {
+		if (session->cgi_type == fastcgi) {
 			close(cgi_info.from_cgi);
 		} else {
 			close(cgi_info.to_cgi);
@@ -618,7 +612,7 @@ int execute_cgi(t_session *session) {
 	cgi_info.timer = session->config->time_for_cgi;
 
 	do {
-		if (use_fast_cgi) {
+		if (session->cgi_type == fastcgi) {
 			result = read_from_fcgi_server(session, &cgi_info);
 		} else {
 			result = read_from_cgi_process(session, &cgi_info);
@@ -626,26 +620,35 @@ int execute_cgi(t_session *session) {
 
 		switch (result) {
 			case cgi_ERROR:
-				log_error(session, "error while executing CGI");
+				log_file_error(session, session->file_on_disk, "error while executing CGI");
 				retval = 500;
 				break;
 			case cgi_TIMEOUT:
-				log_error(session, "CGI timeout");
+				log_file_error(session, session->file_on_disk, "CGI timeout");
 				if (in_body) {
 					retval = rs_DISCONNECT;
 				} else {
 					retval = 500;
 				}
-				if (session->config->kill_timedout_cgi && (use_fast_cgi == false)) {
-					kill(cgi_pid, 15);
-					sleep(1);
-					kill(cgi_pid, 9);
+				if (session->config->kill_timedout_cgi && (session->cgi_type != fastcgi)) {
+					if (kill(cgi_pid, 15) != -1) {
+						sleep(1);
+						kill(cgi_pid, 9);
+					}
 				}
 				break;
 			case cgi_FORCE_QUIT:
 				retval = rs_FORCE_QUIT;
 				break;
 			case cgi_OKE:
+				if (cgi_info.error_len > 0) {
+					/* Error received from CGI
+					 */
+					*(cgi_info.error_buffer + cgi_info.error_len) = '\0';
+					log_file_error(session, session->file_on_disk, cgi_info.error_buffer);
+					cgi_info.error_len = 0;
+				}
+
 				if (cgi_info.input_len > 0) {
 					/* Data received from CGI
 					 */
@@ -691,19 +694,15 @@ int execute_cgi(t_session *session) {
 								while (*line != '\0') {
 									if (*line == '\r') {
 										*line = '\0';
+										break;
 									}
 									line++;
 								}
+
 								if ((retval = str2int(code)) <= 0) {
-									log_error(session, "invalid status code received from CGI");
+									log_file_error(session, session->file_on_disk, "invalid status code received from CGI");
 									retval = 500;
 								}
-								if (retval == 403) {
-									log_error(session, "CGI-wrapper returned 403");
-								}
-								if (send_header(session, retval) == -1) {
-									retval = rs_DISCONNECT;
-								}
 								break;
 							} else if (send_header(session, session->return_code) == -1) {
 								retval = rs_DISCONNECT;
@@ -763,14 +762,6 @@ int execute_cgi(t_session *session) {
 						}
 					}
 				}
-
-				if (cgi_info.error_len > 0) {
-					/* Error received from CGI
-					 */
-					*(cgi_info.error_buffer + cgi_info.error_len) = '\0';
-					log_file_error(session, session->file_on_disk, cgi_info.error_buffer);
-					cgi_info.error_len = 0;
-				}
 				break;
 			case cgi_END_OF_DATA:
 				if (in_body) {
@@ -784,7 +775,7 @@ int execute_cgi(t_session *session) {
 					if (cgi_info.input_len == 0) {
 						log_file_error(session, session->file_on_disk, "no output");
 					} else if (strstr(cgi_info.input_buffer, "\r\n\r\n") == NULL) {
-						log_file_error(session, session->file_on_disk, "non-RFC HTTP header");
+						log_file_error(session, session->file_on_disk, "non-RFC HTTP header ('\\n' instead of '\\r\\n')");
 					} else {
 						log_error(session, "CGI only printed a HTTP header, no content");
 					}
@@ -793,7 +784,7 @@ int execute_cgi(t_session *session) {
 		} // switch
 	} while (retval == 200);
 
-	if (use_fast_cgi) {
+	if (session->cgi_type == fastcgi) {
 		close(cgi_info.from_cgi);
 	} else {
 		close(cgi_info.to_cgi);
@@ -1115,7 +1106,7 @@ int handle_options_request(t_session *session) {
 		return -1;
 	} else if (send_buffer(session, hs_allow, 25) == -1) {
 		return -1;
-	} else if (session->is_cgi_script) {
+	} else if (session->cgi_type != none) {
 		if (send_buffer(session, hs_post, 6) == -1) {
 			return -1;
 		}
@@ -1210,36 +1201,3 @@ int handle_trace_request(t_session *session) {
 
 	return result;
 }
-
-#ifdef HAVE_PLUGIN
-/* Give control to the plugin.
- */
-int handle_plugin(t_session *session) {
-	t_request_info *request_info;
-
-	if ((request_info = (t_request_info*)malloc(sizeof(t_request_info))) != NULL) {
-		request_info->fd = session->client_socket;
-		if ((request_info->fp = fdopen(session->client_socket, "w")) == NULL) {
-			free(request_info);
-			return 500;
-		}
-		request_info->keep_alive = session->keep_alive;
-		request_info->parameters = session->vars;
-		request_info->cookies = get_headerfield("Cookie:", session->headerfields);
-
-		if (send_header(session, 200) == -1) {
-			return 500;
-		}
-		send_buffer(session, NULL, 0);
-		chdir(session->host->website_root);
-		plugin_request(request_info);
-
-		fflush(request_info->fp);
-		free(request_info);
-
-		return 200;
-	} else {
-		return 500;
-	}
-}
-#endif
